@@ -3,7 +3,7 @@
  *
  * See the IPFILTER.LICENCE file for details on licencing.
  *
- * $Id: ip_auth.h,v 2.16.2.2 2006/03/16 06:45:49 darrenr Exp $
+ * $Id: ip_auth.h,v 2.16.2.3 2006/07/14 06:12:05 darrenr Exp $
  *
  */
 #ifndef	__IP_AUTH_H__
@@ -27,7 +27,9 @@ typedef struct  frauth {
 typedef	struct	frauthent  {
 	struct	frentry	fae_fr;
 	struct	frauthent	*fae_next;
+	struct	frauthent	**fae_pnext;
 	u_long	fae_age;
+	int	fae_ref;
 } frauthent_t;
 
 typedef struct  fr_authstat {
@@ -60,7 +62,7 @@ extern	int	fr_authflush __P((void));
 extern	mb_t	**fr_authpkts;
 extern	int	fr_newauth __P((mb_t *, fr_info_t *));
 extern	int	fr_preauthcmd __P((ioctlcmd_t, frentry_t *, frentry_t **));
-extern	int	fr_auth_ioctl __P((caddr_t, ioctlcmd_t, int));
+extern	int	fr_auth_ioctl __P((caddr_t, ioctlcmd_t, int, int, void *));
 extern	int	fr_auth_waiting __P((void));
 
 #endif	/* __IP_AUTH_H__ */
