@@ -5,7 +5,7 @@
  */
 #if !defined(lint)
 static const char sccsid[] = "@(#)ip_fil.c	2.41 6/5/96 (C) 1993-2000 Darren Reed";
-static const char rcsid[] = "@(#)$Id: ip_fil_osf.c,v 2.44.2.20 2006/03/25 13:03:03 darrenr Exp $";
+static const char rcsid[] = "@(#)$Id: ip_fil_osf.c,v 2.44.2.24 2006/10/24 10:21:44 darrenr Exp $";
 #endif
 
 #if defined(KERNEL) || defined(_KERNEL)
@@ -81,6 +81,7 @@ ipfmutex_t	ipf_rw, ipl_mutex, ipf_authmx, ipf_timeoutlock;
 ipfmutex_t	ipf_nat_new, ipf_natio, ipf_stinsert;
 ipfrwlock_t	ipf_mutex, ipf_global, ipf_frag, ipf_tru64, ipf_frcache;
 ipfrwlock_t	ipf_state, ipf_nat, ipf_natfrag, ipf_auth, ipf_ipidfrag;
+ipfrwlock_t	ipf_tokens;
 int		ipf_locks_done = 0;
 
 #if	defined(IPFILTER_LKM)
@@ -94,7 +95,7 @@ char *s;
 #endif /* IPFILTER_LKM */
 
 
-int iplattach()
+int ipfattach()
 {
 	int s, i;
 
@@ -108,6 +109,7 @@ int iplattach()
 	MUTEX_INIT(&ipf_rw, 0);
 	MUTEX_INIT(&ipf_timeoutlock, 0);
 	RWLOCK_INIT(&ipf_ipidfrag, 1);
+	RWLOCK_INIT(&ipf_tokens, 1);
 	ipf_locks_done = 1;
 
 	i = fr_initialise();
@@ -136,7 +138,7 @@ int iplattach()
  * Disable the filter by removing the hooks from the IP input/output
  * stream.
  */
-int ipldetach()
+int ipfdetach()
 {
 	int s;
 
@@ -160,6 +162,7 @@ int ipldetach()
 	if (ipf_locks_done == 1) {
 		MUTEX_DESTROY(&ipf_rw);
 		MUTEX_DESTROY(&ipf_timeoutlock);
+		RW_DESTROY(&ipf_tokens);
 		RW_DESTROY(&ipf_ipidfrag);
 		ipf_locks_done = 0;
 	}
@@ -176,8 +179,8 @@ int cmd;
 caddr_t data;
 int mode;
 {
-	int error = 0, unit = 0, tmp, s;
-	friostat_t fio;
+	int error = 0, unit = 0;
+	SPL_INT(s);
 
 	unit = minor(dev);
 	if ((IPL_LOGMAX < unit) || (unit < 0))
@@ -197,153 +200,12 @@ int mode;
 
 	SPL_NET(s);
 
-	error = fr_ioctlswitch(unit, data, cmd, mode);
+	error = fr_ioctlswitch(unit, data, cmd, mode, curproc->p_uid, curproc);
 	if (error != -1) {
 		SPL_X(s);
 		return error;
 	}
-	error = 0;
 
-	switch (cmd)
-	{
-	case FIONREAD :
-#ifdef IPFILTER_LOG
-		bcopy(&iplused[IPL_LOGIPF], (caddr_t)data,
-		         sizeof(iplused[IPL_LOGIPF]));
-#endif
-		break;
-	case SIOCFRENB :
-		if (!(mode & FWRITE))
-			error = EPERM;
-		else {
-			RWLOCK_EXIT(&ipf_tru64);
-			WRITE_ENTER(&ipf_tru64);
-			bcopy(data, &tmp, sizeof(tmp));
-			if (tmp) {
-				if (fr_running > 0)
-					error = 0;
-				else
-					error = iplattach();
-				if (error == 0)
-					fr_running = 1;
-				else
-					(void) ipldetach();
-			} else {
-				error = ipldetach();
-				if (error == 0)
-					fr_running = -1;
-			}
-		}
-		break;
-	case SIOCIPFSET :
-		if (!(mode & FWRITE)) {
-			error = EPERM;
-			break;
-		}
-	case SIOCIPFGETNEXT :
-	case SIOCIPFGET :
-		error = fr_ipftune(cmd, data);
-		break;
-	case SIOCSETFF :
-		if (!(mode & FWRITE))
-			error = EPERM;
-		else
-			bcopy(data, &fr_flags, sizeof(fr_flags));
-		break;
-	case SIOCGETFF :
-		bcopy(&fr_flags, data, sizeof(fr_flags));
-		break;
-	case SIOCFUNCL :
-		error = fr_resolvefunc(data);
-		break;
-	case SIOCINAFR :
-	case SIOCRMAFR :
-	case SIOCADAFR :
-	case SIOCZRLST :
-		if (!(mode & FWRITE))
-			error = EPERM;
-		else
-			error = frrequest(unit, cmd, data, fr_active, 1);
-		break;
-	case SIOCINIFR :
-	case SIOCRMIFR :
-	case SIOCADIFR :
-		if (!(mode & FWRITE))
-			error = EPERM;
-		else
-			error = frrequest(unit, cmd, data, 1 - fr_active, 1);
-		break;
-	case SIOCSWAPA :
-		if (!(mode & FWRITE))
-			error = EPERM;
-		else {
-			bzero((char *)frcache, sizeof(frcache[0]) * 2);
-			*(u_int *)data = fr_active;
-			fr_active = 1 - fr_active;
-		}
-		break;
-	case SIOCGETFS :
-		fr_getstat(&fio);
-		error = fr_outobj(data, &fio, IPFOBJ_IPFSTAT);
-		break;
-	case	SIOCFRZST :
-		if (!(mode & FWRITE))
-			error = EPERM;
-		else
-			error = fr_zerostats(data);
-		break;
-	case	SIOCIPFFL :
-		if (!(mode & FWRITE))
-			error = EPERM;
-		else {
-			bcopy(data, &tmp, sizeof(tmp));
-			tmp = frflush(unit, 4, tmp);
-			bcopy(&tmp, data, sizeof(tmp));
-		}
-		break;
-#ifdef USE_INET6
-	case	SIOCIPFL6 :
-		if (!(mode & FWRITE))
-			error = EPERM;
-		else {
-			bcopy(data, &tmp, sizeof(tmp));
-			tmp = frflush(unit, 6, tmp);
-			bcopy(&tmp, data, sizeof(tmp));
-		}
-		break;
-#endif
-	case SIOCSTLCK :
-		error = COPYIN(data, &tmp, sizeof(tmp));
-		if (error == 0) {
-			fr_state_lock = tmp;
-			fr_nat_lock = tmp;
-			fr_frag_lock = tmp;
-			fr_auth_lock = tmp;
-		} else
-			error = EFAULT;
-		break;
-#ifdef	IPFILTER_LOG
-	case	SIOCIPFFB :
-		if (!(mode & FWRITE))
-			error = EPERM;
-		else
-			*(int *)data = ipflog_clear(unit);
-		break;
-#endif /* IPFILTER_LOG */
-	case SIOCGFRST :
-		error = fr_outobj(data, fr_fragstats(), IPFOBJ_FRAGSTAT);
-		break;
-	case SIOCFRSYN :
-		if (!(mode & FWRITE))
-			error = EPERM;
-		else {
-			frsync(NULL);
-		}
-		break;
-	default :
-		error = EINVAL;
-		break;
-	}
 	SPL_X(s);
 	return error;
 }
@@ -428,6 +290,9 @@ int iplread(dev, uio)
 dev_t dev;
 register struct uio *uio;
 {
+	if (fr_running < 1)
+		return EIO;
+
 #ifdef IPFILTER_LOG
 	return ipflog_read(minor(dev), uio);
 #else
@@ -790,7 +655,7 @@ void iplinit __P((void));
 
 void iplinit()
 {
-	if (iplattach() != 0)
+	if (ipfattach() != 0)
 		printf("IP Filter failed to attach\n");
 	ip_init();
 }
@@ -936,6 +801,7 @@ frdest_t *fdp;
 		case 0 :
 			break;
 		case 1 :
+			fr_natderef((nat_t **)&fin->fin_nat);
 			ip->ip_sum = 0;
 			break;
 		case -1 :
@@ -1408,3 +1274,30 @@ int len;
 		fin->fin_flx |= FI_COALESCE;
 	return ip;
 }
+
+
+int ipf_inject(fin, m)
+fr_info_t *fin;
+mb_t *m;
+{
+	int error;
+
+	if (fin->fin_out == 0) {
+		struct ifqueue *ifq;
+
+		ifq = &ipintrq;
+
+		if (IF_QFULL(ifq)) {
+			IF_DROP(ifq);
+			FREE_MB_T(m);
+			error = ENOBUFS;
+		} else {
+			IF_ENQUEUE(ifq, m);
+			error = 0;
+		}
+	} else {
+		error = ip_output(m, NULL, NULL, IP_FORWARDING, NULL);
+	}
+
+	return error;
+}
