@@ -4,7 +4,7 @@
  * See the IPFILTER.LICENCE file for details on licencing.
  *
  * @(#)ip_frag.h	1.5 3/24/96
- * $Id: ip_frag.h,v 2.23.2.4 2006/07/22 20:55:30 darrenr Exp $
+ * $Id: ip_frag.h,v 2.23.2.5 2006/12/23 11:11:47 darrenr Exp $
  */
 
 #ifndef	__IP_FRAG_H__
@@ -16,6 +16,16 @@ typedef	struct	ipfr	{
 	struct	ipfr	*ipfr_hnext, **ipfr_hprev;
 	struct	ipfr	*ipfr_next, **ipfr_prev;
 	void	*ipfr_data;
+	frentry_t *ipfr_rule;
+	u_long	ipfr_ttl;
+	int	ipfr_ref;
+	u_short	ipfr_off;
+	u_short	ipfr_seen0;
+	/*
+	 * All of the fields, from ipfr_ifp to ipfr_pass, are compared
+	 * using bcmp to see if an identical entry is present.  It is
+	 * therefore important for this set to remain together.
+	 */
 	void	*ipfr_ifp;
 	struct	in_addr	ipfr_src;
 	struct	in_addr	ipfr_dst;
@@ -26,11 +36,6 @@ typedef	struct	ipfr	{
 	u_char	ipfr_p;
 	u_char	ipfr_tos;
 	u_32_t	ipfr_pass;
-	u_short	ipfr_off;
-	u_char	ipfr_ttl;
-	u_char	ipfr_seen0;
-	frentry_t *ipfr_rule;
-	int	ipfr_ref;
 } ipfr_t;
 
 
