@@ -51,7 +51,7 @@ struct file;
 /* END OF INCLUDES */
 
 #if !defined(lint)
-static const char rcsid[] = "@(#)$Id: ip_htable.c,v 2.34.2.6 2006/08/26 06:28:56 darrenr Exp $";
+static const char rcsid[] = "@(#)$Id: ip_htable.c,v 2.34.2.8 2007/01/17 03:16:53 darrenr Exp $";
 #endif
 
 #ifdef	IPFILTER_LOOKUP
@@ -101,30 +101,36 @@ iplookupop_t *op;
 	char name[FR_GROUPLEN];
 	int err, i, unit;
 
-	KMALLOC(iph, iphtable_t *);
-	if (iph == NULL) {
-		ipht_nomem[op->iplo_unit]++;
-		return ENOMEM;
-	}
-
-	err = COPYIN(op->iplo_struct, iph, sizeof(*iph));
-	if (err != 0) {
-		KFREE(iph);
-		return EFAULT;
-	}
-
 	unit = op->iplo_unit;
+	if ((op->iplo_arg & IPHASH_ANON) == 0)
+		iph = fr_existshtable(unit, op->iplo_name);
+	else
+		iph = NULL;
+
+	if (iph == NULL) {
+		KMALLOC(iph, iphtable_t *);
+		if (iph == NULL) {
+			ipht_nomem[op->iplo_unit]++;
+			return ENOMEM;
+		}
+		err = COPYIN(op->iplo_struct, iph, sizeof(*iph));
+		if (err != 0) {
+			KFREE(iph);
+			return EFAULT;
+		}
+	} else {
+		if ((iph->iph_flags & IPHASH_DELETE) == 0)
+			return EEXIST;
+	}
+
 	if (iph->iph_unit != unit) {
-		KFREE(iph);
+		if ((iph->iph_flags & IPHASH_DELETE) == 0) {
+			KFREE(iph);
+		}
 		return EINVAL;
 	}
 
-	if ((op->iplo_arg & IPHASH_ANON) == 0) {
-		if (fr_findhtable(unit, op->iplo_name) != NULL) {
-			KFREE(iph);
-			return EEXIST;
-		}
-	} else {
+	if ((op->iplo_arg & IPHASH_ANON) != 0) {
 		i = IPHASH_ANON;
 		do {
 			i++;
@@ -145,26 +151,33 @@ iplookupop_t *op;
 		iph->iph_type |= IPHASH_ANON;
 	}
 
-	KMALLOCS(iph->iph_table, iphtent_t **,
-		 iph->iph_size * sizeof(*iph->iph_table));
-	if (iph->iph_table == NULL) {
-		KFREE(iph);
-		ipht_nomem[unit]++;
-		return ENOMEM;
+	if ((iph->iph_flags & IPHASH_DELETE) == 0) {
+		KMALLOCS(iph->iph_table, iphtent_t **,
+			 iph->iph_size * sizeof(*iph->iph_table));
+		if (iph->iph_table == NULL) {
+			if ((iph->iph_flags & IPHASH_DELETE) == 0) {
+				KFREE(iph);
+			}
+			ipht_nomem[unit]++;
+			return ENOMEM;
+		}
+
+		bzero((char *)iph->iph_table,
+		      iph->iph_size * sizeof(*iph->iph_table));
+		iph->iph_masks = 0;
+		iph->iph_list = NULL;
+
+		iph->iph_ref = 1;
+		iph->iph_next = ipf_htables[unit];
+		iph->iph_pnext = &ipf_htables[unit];
+		if (ipf_htables[unit] != NULL)
+			ipf_htables[unit]->iph_pnext = &iph->iph_next;
+		ipf_htables[unit] = iph;
+
+		ipf_nhtables[unit]++;
 	}
 
-	bzero((char *)iph->iph_table, iph->iph_size * sizeof(*iph->iph_table));
-	iph->iph_masks = 0;
-	iph->iph_list = NULL;
-
-	iph->iph_ref = 1;
-	iph->iph_next = ipf_htables[unit];
-	iph->iph_pnext = &ipf_htables[unit];
-	if (ipf_htables[unit] != NULL)
-		ipf_htables[unit]->iph_pnext = &iph->iph_next;
-	ipf_htables[unit] = iph;
-
-	ipf_nhtables[unit]++;
+	iph->iph_flags &= ~IPHASH_DELETE;
 
 	return 0;
 }
@@ -172,21 +185,24 @@ iplookupop_t *op;
 
 /*
  */
-int fr_removehtable(op)
-iplookupop_t *op;
+int fr_removehtable(unit, name)
+int unit;
+char *name;
 {
 	iphtable_t *iph;
 
-	iph = fr_findhtable(op->iplo_unit, op->iplo_name);
+	iph = fr_findhtable(unit, name);
 	if (iph == NULL)
 		return ESRCH;
 
-	if (iph->iph_unit != op->iplo_unit) {
+	if (iph->iph_unit != unit) {
 		return EINVAL;
 	}
 
 	if (iph->iph_ref != 0) {
-		return EBUSY;
+		(void) fr_clearhtable(iph);
+		iph->iph_flags |= IPHASH_DELETE;
+		return 0;
 	}
 
 	fr_delhtable(iph);
@@ -195,14 +211,24 @@ iplookupop_t *op;
 }
 
 
-void fr_delhtable(iph)
+int fr_clearhtable(iph)
 iphtable_t *iph;
 {
 	iphtent_t *ipe;
 
 	while ((ipe = iph->iph_list) != NULL)
 		if (fr_delhtent(iph, ipe) != 0)
-			return;
+			return 1;
+	return 0;
+}
+
+
+int fr_delhtable(iph)
+iphtable_t *iph;
+{
+
+	if (fr_clearhtable(iph) != 0)
+		return 1;
 
 	if (iph->iph_pnext != NULL)
 		*iph->iph_pnext = iph->iph_next;
@@ -211,7 +237,7 @@ iphtable_t *iph;
 
 	ipf_nhtables[iph->iph_unit]--;
 
-	fr_derefhtable(iph);
+	return fr_derefhtable(iph);
 }
 
 
@@ -246,36 +272,45 @@ iphtent_t *ipe;
 		break;
 	}
 
-	fr_derefhtent(ipe);
-
-	return 0;
+	return fr_derefhtent(ipe);
 }
 
 
-void fr_derefhtable(iph)
+int fr_derefhtable(iph)
 iphtable_t *iph;
 {
+	int refs;
+
 	iph->iph_ref--;
+	refs = iph->iph_ref;
+
 	if (iph->iph_ref == 0) {
 		KFREES(iph->iph_table, iph->iph_size * sizeof(*iph->iph_table));
 		KFREE(iph);
 	}
+
+	return refs;
 }
 
 
-void fr_derefhtent(ipe)
+int fr_derefhtent(ipe)
 iphtent_t *ipe;
 {
+
 	ipe->ipe_ref--;
 	if (ipe->ipe_ref == 0) {
 		ipf_nhtnodes[ipe->ipe_unit]--;
 
 		KFREE(ipe);
+
+		return 0;
 	}
+
+	return ipe->ipe_ref;
 }
 
 
-iphtable_t *fr_findhtable(unit, name)
+iphtable_t *fr_existshtable(unit, name)
 int unit;
 char *name;
 {
@@ -288,6 +323,20 @@ char *name;
 }
 
 
+iphtable_t *fr_findhtable(unit, name)
+int unit;
+char *name;
+{
+	iphtable_t *iph;
+
+	iph = fr_existshtable(unit, name);
+	if ((iph->iph_flags & IPHASH_DELETE) == 0)
+		return iph;
+
+	return NULL;
+}
+
+
 size_t fr_flushhtable(op)
 iplookupflush_t *op;
 {
@@ -300,8 +349,11 @@ iplookupflush_t *op;
 	for (i = 0; i <= IPL_LOGMAX; i++) {
 		if (op->iplf_unit == i || op->iplf_unit == IPL_LOGALL) {
 			while ((iph = ipf_htables[i]) != NULL) {
-				fr_delhtable(iph);
-				freed++;
+				if (fr_delhtable(iph) == 0) {
+					freed++;
+				} else {
+					iph->iph_flags |= IPHASH_DELETE;
+				}
 			}
 		}
 	}
