@@ -3,7 +3,7 @@
  *
  * See the IPFILTER.LICENCE file for details on licencing.
  *
- * $Id: printfraginfo.c,v 1.1.2.4 2006/07/14 06:12:25 darrenr Exp $
+ * $Id: printfraginfo.c,v 1.1.2.5 2006/12/25 15:10:37 darrenr Exp $
  */
 #include "ipf.h"
 #include "kmem.h"
@@ -22,8 +22,7 @@ struct ipfr *ifr;
 		    sizeof(fr)) == -1)
 		return;
 */
-	printf("%s id %d ttl %d pr %d seen0 %d ref %d tos %#02x = %#x\n",
-		hostname(4, &ifr->ipfr_dst), ifr->ipfr_id, ifr->ipfr_seen0,
-		ifr->ipfr_ttl, ifr->ipfr_p, ifr->ipfr_ref, ifr->ipfr_tos,
-		0);
+	printf("%s id %d ttl %ld pr %d seen0 %d ref %d tos %#02x\n",
+		hostname(4, &ifr->ipfr_dst), ifr->ipfr_id, ifr->ipfr_ttl,
+		ifr->ipfr_p, ifr->ipfr_seen0, ifr->ipfr_ref, ifr->ipfr_tos);
 }
