@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 by Darren Reed.
+ * Copyright (C) 2002-2005 by Darren Reed.
  *
  * See the IPFILTER.LICENCE file for details on licencing.
  */
@@ -21,11 +21,11 @@ int opts;
 		printip((u_32_t *)&np->ipn_addr.adf_addr.in4);
 		printmask((u_32_t *)&np->ipn_mask.adf_addr);
 	} else {
-		PRINTF("\t\t%s%s", np->ipn_info ? "! " : "",
+		PRINTF("\tAddress: %s%s", np->ipn_info ? "! " : "",
 			inet_ntoa(np->ipn_addr.adf_addr.in4));
 		printmask((u_32_t *)&np->ipn_mask.adf_addr);
-		PRINTF("\n\t\tHits %lu\tName %s\n",
-			np->ipn_hits, np->ipn_name);
+		PRINTF("\t\tHits %lu\tName %s\tRef %d\n",
+			np->ipn_hits, np->ipn_name, np->ipn_ref);
 	}
 	return np->ipn_next;
 }
