@@ -176,7 +176,7 @@ static int ipl_unload()
         if (fr_refcnt)
                 error = EBUSY;
 	else if (fr_running >= 0)
-		error = ipldetach();
+		error = ipfdetach();
 
 	if (error == 0) {
 		fr_running = -2;
@@ -201,7 +201,7 @@ static int ipl_load()
 	 */
 	(void)ipl_remove();
 
-	error = iplattach();
+	error = ipfattach();
 
 	for (i = 0; (error == 0) && (name = ipf_devfiles[i]); i++) {
 		NDINIT(&nd, CREATE, LOCKPARENT, UIO_SYSSPACE, name, curproc);
