@@ -1,3 +1,8 @@
+/*
+ * Copyright (C) 2002-2004 by Darren Reed.
+ *
+ * See the IPFILTER.LICENCE file for details on licencing.
+ */
 
 typedef	struct	wordtab	{
 	char	*w_word;
@@ -21,9 +26,11 @@ typedef	struct	wordtab	{
 #define	YYBUFSIZ	8192
 
 extern	wordtab_t	*yysettab __P((wordtab_t *));
+extern	void		yysetdict __P((wordtab_t *));
 extern	int		yylex __P((void));
 extern	void		yyerror __P((char *));
 extern	char		*yykeytostr __P((int));
+extern	void		yyresetdict __P((void));
 
 extern	FILE	*yyin;
 extern	int	yylineNum;
